@@ -32,6 +32,7 @@
 #include "gr-category-tile.h"
 #include "gr-image.h"
 #include "gr-app.h"
+#include "gr-settings.h"
 
 struct _GrListPage
 {
@@ -60,6 +61,8 @@ struct _GrListPage
         GtkWidget *heading;
         GtkWidget *diet_description;
 
+        gboolean show_shared;
+
         int count;
         GrRecipeSearch *search;
 };
@@ -149,6 +152,7 @@ search_hits_added (GrRecipeSearch *search,
                 GtkWidget *tile;
 
                 tile = gr_recipe_tile_new (recipe);
+                gr_recipe_tile_set_show_shared (GR_RECIPE_TILE (tile), page->show_shared);
                 gtk_widget_show (tile);
                 gtk_container_add (GTK_CONTAINER (page->flow_box), tile);
 
@@ -190,6 +194,64 @@ search_finished (GrRecipeSearch *search,
                                           page->count > 0 ? "list" : "empty");
 }
 
+static int
+name_sort_func (GtkFlowBoxChild *child1,
+                GtkFlowBoxChild *child2,
+                gpointer         data)
+{
+        GtkWidget *tile1 = gtk_bin_get_child (GTK_BIN (child1));
+        GtkWidget *tile2 = gtk_bin_get_child (GTK_BIN (child2));
+        GrRecipe *recipe1 = gr_recipe_tile_get_recipe (GR_RECIPE_TILE (tile1));
+        GrRecipe *recipe2 = gr_recipe_tile_get_recipe (GR_RECIPE_TILE (tile2));
+        const char *name1 = gr_recipe_get_name (recipe1);
+        const char *name2 = gr_recipe_get_name (recipe2);
+
+        return strcmp (name1, name2);
+}
+
+static int
+date_sort_func (GtkFlowBoxChild *child1,
+                GtkFlowBoxChild *child2,
+                gpointer         data)
+{
+        GtkWidget *tile1 = gtk_bin_get_child (GTK_BIN (child1));
+        GtkWidget *tile2 = gtk_bin_get_child (GTK_BIN (child2));
+        GrRecipe *recipe1 = gr_recipe_tile_get_recipe (GR_RECIPE_TILE (tile1));
+        GrRecipe *recipe2 = gr_recipe_tile_get_recipe (GR_RECIPE_TILE (tile2));
+        GDateTime *mtime1 = gr_recipe_get_mtime (recipe1);
+        GDateTime *mtime2 = gr_recipe_get_mtime (recipe2);
+
+        return g_date_time_compare (mtime2, mtime1);
+}
+
+static void
+gr_list_page_set_sort (GrListPage *page,
+                       GrSortKey   sort)
+{
+        switch (sort) {
+        case SORT_BY_NAME:
+                gtk_flow_box_set_sort_func (GTK_FLOW_BOX (page->flow_box), name_sort_func, page, NULL);
+                break;
+        case SORT_BY_RECENCY:
+                gtk_flow_box_set_sort_func (GTK_FLOW_BOX (page->flow_box), date_sort_func, page, NULL);
+                break;
+        default:
+                g_assert_not_reached ();
+        }
+}
+
+static void
+sort_key_changed (GrListPage *page)
+{
+        GrSortKey sort;
+
+        if (!gtk_widget_get_visible (GTK_WIDGET (page)))
+                return;
+
+        sort = g_settings_get_enum (gr_settings_get (), "sort-key");
+        gr_list_page_set_sort (page, sort);
+}
+
 static void
 gr_list_page_init (GrListPage *page)
 {
@@ -202,6 +264,9 @@ gr_list_page_init (GrListPage *page)
         g_signal_connect (page->search, "hits-added", G_CALLBACK (search_hits_added), page);
         g_signal_connect (page->search, "hits-removed", G_CALLBACK (search_hits_removed), page);
         g_signal_connect (page->search, "finished", G_CALLBACK (search_finished), page);
+
+        g_signal_connect_swapped (gr_settings_get (), "changed::sort-key", G_CALLBACK (sort_key_changed), page);
+        g_signal_connect (page, "notify::visible", G_CALLBACK (sort_key_changed), NULL);
 }
 
 static void
@@ -282,6 +347,8 @@ gr_list_page_populate_from_diet (GrListPage *self,
         char *tmp;
         g_autofree char *term = NULL;
 
+        self->show_shared = FALSE;
+
         clear_data (self);
         self->diet = diet;
 
@@ -304,9 +371,17 @@ gr_list_page_populate_from_diet (GrListPage *self,
         gr_recipe_search_set_query (self->search, term);
 }
 
+void
+gr_list_page_set_show_shared (GrListPage *self,
+                              gboolean    show_shared)
+{
+        self->show_shared = show_shared;
+}
+
 void
 gr_list_page_populate_from_chef (GrListPage *self,
-                                 GrChef     *chef)
+                                 GrChef     *chef,
+                                 gboolean    show_shared)
 {
         GrRecipeStore *store;
         const char *id;
@@ -317,6 +392,8 @@ gr_list_page_populate_from_chef (GrListPage *self,
         char *tmp;
         g_autofree char *term = NULL;
 
+        self->show_shared = show_shared;
+
         g_object_ref (chef);
         clear_data (self);
         self->chef = chef;
@@ -374,6 +451,8 @@ gr_list_page_populate_from_season (GrListPage *self,
         char *tmp;
         g_autofree char *term = NULL;
 
+        self->show_shared = FALSE;
+
         tmp = g_strdup (season);
         clear_data (self);
         self->season = tmp;
@@ -397,6 +476,8 @@ gr_list_page_populate_from_season (GrListPage *self,
 void
 gr_list_page_populate_from_favorites (GrListPage *self)
 {
+        self->show_shared = FALSE;
+
         clear_data (self);
         self->favorites = TRUE;
 
@@ -416,6 +497,8 @@ gr_list_page_populate_from_favorites (GrListPage *self)
 void
 gr_list_page_populate_from_all (GrListPage *self)
 {
+        self->show_shared = FALSE;
+
         clear_data (self);
         self->all = TRUE;
 
@@ -441,6 +524,8 @@ gr_list_page_populate_from_new (GrListPage *self)
         g_autofree char *query = NULL;
         const char *terms[2];
 
+        self->show_shared = FALSE;
+
         clear_data (self);
         self->new = TRUE;
 
@@ -471,6 +556,8 @@ gr_list_page_populate_from_list (GrListPage *self,
         GList *l;
         gboolean empty;
 
+        self->show_shared = FALSE;
+
         store = gr_recipe_store_get ();
 
         recipes = g_list_copy_deep (recipes, (GCopyFunc)g_object_ref, NULL);
@@ -510,7 +597,7 @@ void
 gr_list_page_repopulate (GrListPage *page)
 {
         if (page->chef)
-                gr_list_page_populate_from_chef (page, page->chef);
+                gr_list_page_populate_from_chef (page, page->chef, page->show_shared);
         else if (page->diet)
                 gr_list_page_populate_from_diet (page, page->diet);
         else if (page->favorites)
