@@ -57,7 +57,6 @@ struct _GrRecipe
         char *category;
         char *prep_time;
         char *cook_time;
-        int serves;
         char *ingredients;
         char *instructions;
         char *notes;
@@ -79,6 +78,9 @@ struct _GrRecipe
         char *translated_description;
         char *translated_instructions;
         char *translated_notes;
+
+        double yield;
+        char *yield_unit;
 };
 
 G_DEFINE_TYPE (GrRecipe, gr_recipe, G_TYPE_OBJECT)
@@ -96,7 +98,8 @@ enum {
         PROP_CATEGORY,
         PROP_PREP_TIME,
         PROP_COOK_TIME,
-        PROP_SERVES,
+        PROP_YIELD,
+        PROP_YIELD_UNIT,
         PROP_INGREDIENTS,
         PROP_INSTRUCTIONS,
         PROP_SPICINESS,
@@ -141,6 +144,8 @@ gr_recipe_finalize (GObject *object)
         g_free (self->translated_instructions);
         g_free (self->translated_notes);
 
+        g_free (self->yield_unit);
+
         G_OBJECT_CLASS (gr_recipe_parent_class)->finalize (object);
 }
 
@@ -197,10 +202,6 @@ gr_recipe_get_property (GObject    *object,
                 g_value_set_string (value, self->cook_time);
                 break;
 
-        case PROP_SERVES:
-                g_value_set_int (value, self->serves);
-                break;
-
         case PROP_SPICINESS:
                 g_value_set_int (value, self->spiciness);
                 break;
@@ -237,6 +238,14 @@ gr_recipe_get_property (GObject    *object,
                 g_value_set_boolean (value, self->contributed);
                 break;
 
+        case PROP_YIELD:
+                g_value_set_double (value, self->yield);
+                break;
+
+        case PROP_YIELD_UNIT:
+                g_value_set_string (value, self->yield_unit);
+                break;
+
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         }
@@ -341,10 +350,6 @@ gr_recipe_set_property (GObject      *object,
                 self->cook_time = g_value_dup_string (value);
                 break;
 
-        case PROP_SERVES:
-                self->serves = g_value_get_int (value);
-                break;
-
         case PROP_SPICINESS:
                 self->spiciness = g_value_get_int (value);
                 break;
@@ -403,6 +408,15 @@ gr_recipe_set_property (GObject      *object,
                 self->contributed = g_value_get_boolean (value);
                 break;
 
+        case PROP_YIELD:
+                self->yield = g_value_get_double (value);
+                break;
+
+        case PROP_YIELD_UNIT:
+                g_free (self->yield_unit);
+                self->yield_unit = g_value_dup_string (value);
+                break;
+
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         }
@@ -478,11 +492,6 @@ gr_recipe_class_init (GrRecipeClass *klass)
                                      G_PARAM_READWRITE);
         g_object_class_install_property (object_class, PROP_COOK_TIME, pspec);
 
-        pspec = g_param_spec_int ("serves", NULL, NULL,
-                                  0, 100, 0,
-                                  G_PARAM_READWRITE);
-        g_object_class_install_property (object_class, PROP_SERVES, pspec);
-
         pspec = g_param_spec_string ("ingredients", NULL, NULL,
                                      NULL,
                                      G_PARAM_READWRITE);
@@ -522,6 +531,16 @@ gr_recipe_class_init (GrRecipeClass *klass)
                                       FALSE,
                                       G_PARAM_READWRITE);
         g_object_class_install_property (object_class, PROP_CONTRIBUTED, pspec);
+
+        pspec = g_param_spec_double ("yield", NULL, NULL,
+                                     0.0, G_MAXDOUBLE, 0.0,
+                                     G_PARAM_READWRITE);
+        g_object_class_install_property (object_class, PROP_YIELD, pspec);
+
+        pspec = g_param_spec_string ("yield-unit", NULL, NULL,
+                                     NULL,
+                                     G_PARAM_READWRITE);
+        g_object_class_install_property (object_class, PROP_YIELD_UNIT, pspec);
 }
 
 static void
@@ -530,6 +549,7 @@ gr_recipe_init (GrRecipe *self)
         self->ctime = g_date_time_new_now_utc ();
         self->mtime = g_date_time_new_now_utc ();
         self->images = gr_image_array_new ();
+        self->yield = 0.0;
 }
 
 GrRecipe *
@@ -580,12 +600,6 @@ gr_recipe_get_translated_notes (GrRecipe *recipe)
         return recipe->translated_notes;
 }
 
-int
-gr_recipe_get_serves (GrRecipe *recipe)
-{
-        return recipe->serves;
-}
-
 const char *
 gr_recipe_get_cuisine (GrRecipe *recipe)
 {
@@ -707,6 +721,18 @@ gr_recipe_get_images (GrRecipe *recipe)
         return recipe->images;
 }
 
+double
+gr_recipe_get_yield (GrRecipe *recipe)
+{
+        return recipe->yield;
+}
+
+const char *
+gr_recipe_get_yield_unit (GrRecipe *recipe)
+{
+        return recipe->yield_unit;
+}
+
 /* terms are assumed to be g_utf8_casefold'ed where appropriate */
 gboolean
 gr_recipe_matches (GrRecipe    *recipe,
