@@ -25,7 +25,7 @@
 
 #include "gr-shopping-list-formatter.h"
 #include "gr-ingredients-list.h"
-#include "gr-images.h"
+#include "gr-image.h"
 #include "gr-chef.h"
 #include "gr-recipe-store.h"
 #include "gr-utils.h"
