@@ -24,9 +24,9 @@
 #include <gtk/gtk.h>
 
 #include "gr-shopping-page.h"
+#include "gr-shopping-tile.h"
 #include "gr-recipe-store.h"
 #include "gr-recipe.h"
-#include "gr-recipe-small-tile.h"
 #include "gr-utils.h"
 #include "gr-ingredients-list.h"
 #include "gr-window.h"
@@ -34,6 +34,8 @@
 #include "gr-shopping-list-printer.h"
 #include "gr-shopping-list-formatter.h"
 #include "gr-mail.h"
+#include "gr-convert-units.h"
+#include "gr-shopping-list-exporter.h"
 
 
 struct _GrShoppingPage
@@ -50,6 +52,7 @@ struct _GrShoppingPage
         int ingredient_count;
         int recipe_count;
         GrRecipeSearch *search;
+        GrShoppingListExporter *exporter;
 
         GtkSizeGroup *group;
         GHashTable *ingredients;
@@ -137,8 +140,8 @@ recount_recipes (GrShoppingPage *page)
 }
 
 typedef struct {
-        GrNumber amount;
-        char *unit;
+        double amount;
+        GrUnit unit;
 } Unit;
 
 typedef struct {
@@ -147,14 +150,6 @@ typedef struct {
         gboolean removed;
 } Ingredient;
 
-static void
-clear_unit (gpointer data)
-{
-        Unit *unit = data;
-
-        g_free (unit->unit);
-}
-
 static Ingredient *
 ingredient_new (const char *ingredient)
 {
@@ -166,7 +161,6 @@ ingredient_new (const char *ingredient)
         ing = g_new0 (Ingredient, 1);
         ing->ingredient = g_strdup (ingredient);
         ing->units = g_array_new (FALSE, TRUE, sizeof (Unit));
-        g_array_set_clear_func (ing->units, clear_unit);
 
         ing->removed = gr_recipe_store_not_shopping_ingredient (store, ingredient);
 
@@ -191,8 +185,8 @@ ingredient_clear (Ingredient *ing)
 
 static void
 ingredient_add (Ingredient *ing,
-                GrNumber   *amount,
-                const char *unit)
+                double      amount,
+                GrUnit      unit)
 {
         int i;
         Unit nu;
@@ -200,14 +194,14 @@ ingredient_add (Ingredient *ing,
         for (i = 0; i < ing->units->len; i++) {
                 Unit *u = &g_array_index (ing->units, Unit, i);
 
-                if (g_strcmp0 (u->unit, unit) == 0) {
-                        gr_number_add (&u->amount, amount, &u->amount);
+                if (u->unit == unit) {
+                        u->amount += amount;
                         return;
                 }
         }
 
-        nu.amount = *amount;
-        nu.unit = g_strdup (unit);
+        nu.amount = amount;
+        nu.unit = unit;
 
         g_array_append_val (ing->units, nu);
 }
@@ -217,21 +211,41 @@ ingredient_format_unit (Ingredient *ing)
 {
         g_autoptr(GString) s = NULL;
         int i;
-
+        GrPreferredUnit user_volume_unit = gr_convert_get_volume_unit();
+        GrPreferredUnit user_weight_unit = gr_convert_get_weight_unit();
+        GrUnit u1;
+        double a1;
+        GrDimension dimension;
         s = g_string_new ("");
 
         for (i = 0; i < ing->units->len; i++) {
-                Unit *u = &g_array_index (ing->units, Unit, i);
-                g_autofree char *num = NULL;
-                if (s->len > 0)
-                        g_string_append (s, ", ");
-                num = gr_number_format (&(u->amount));
-                g_string_append (s, num);
-                g_string_append (s, " ");
-                if (u->unit)
-                        g_string_append (s, u->unit);
+                Unit *unit = &g_array_index (ing->units, Unit, i);
+                double a = unit->amount;
+                GrUnit u = unit->unit;
+
+                dimension = gr_unit_get_dimension (u);
+
+                if (dimension == GR_DIMENSION_VOLUME) {
+                        gr_convert_volume (&a, &u, user_volume_unit);
+                }
+                else if (dimension == GR_DIMENSION_MASS) {
+                        gr_convert_weight (&a, &u, user_weight_unit);
+                }
+
+                if (i == 0) {
+                        u1 = u;
+                        a1 = a;
+                }
+                else if (u == u1) {
+                        a1 += a;
+                }
+                else {
+                        g_error ("conversion yielded different units...why...");
+                }
         }
 
+        gr_convert_format (s, a1, u1);
+
         return g_strdup (s->str);
 }
 
@@ -434,8 +448,8 @@ removed_row_activated (GtkListBox     *list,
 
 static void
 add_ingredient (GrShoppingPage *page,
-                GrNumber *amount,
-                const char *unit,
+                double      amount,
+                GrUnit      unit,
                 const char *ingredient)
 {
         Ingredient *ing;
@@ -452,7 +466,7 @@ add_ingredient (GrShoppingPage *page,
 static void
 collect_ingredients_from_recipe (GrShoppingPage *page,
                                  GrRecipe       *recipe,
-                                 int             serves)
+                                 double          yield)
 {
         g_autoptr(GrIngredientsList) il = NULL;
         g_autofree char **seg = NULL;
@@ -464,12 +478,11 @@ collect_ingredients_from_recipe (GrShoppingPage *page,
                 g_auto(GStrv) ing = NULL;
                 ing = gr_ingredients_list_get_ingredients (il, seg[i]);
                 for (j = 0; ing[j]; j++) {
-                        const char *unit;
-                        GrNumber *amount, *num;
+                        GrUnit unit;
+                        double amount;
+
                         amount = gr_ingredients_list_get_amount (il, seg[i], ing[j]);
-                        num = gr_number_new_fraction (serves, gr_recipe_get_serves (recipe));
-                        gr_number_multiply (num, amount, amount);
-                        g_free (num);
+                        amount = amount * yield / gr_recipe_get_yield (recipe);
                         unit = gr_ingredients_list_get_unit (il, seg[i], ing[j]);
                         add_ingredient (page, amount, unit, ing[j]);
                 }
@@ -493,11 +506,11 @@ collect_ingredients (GrShoppingPage *page)
         for (l = children; l; l = l->next) {
                 GtkWidget *tile = gtk_bin_get_child (GTK_BIN (l->data));
                 GrRecipe *recipe;
-                int serves;
+                double yield;
 
-                recipe = gr_recipe_small_tile_get_recipe (GR_RECIPE_SMALL_TILE (tile));
-                serves = gr_recipe_small_tile_get_serves (GR_RECIPE_SMALL_TILE (tile));
-                collect_ingredients_from_recipe (page, recipe, serves);
+                recipe = gr_shopping_tile_get_recipe (GR_SHOPPING_TILE (tile));
+                yield = gr_shopping_tile_get_yield (GR_SHOPPING_TILE (tile));
+                collect_ingredients_from_recipe (page, recipe, yield);
         }
         g_list_free (children);
 
@@ -521,24 +534,24 @@ collect_ingredients (GrShoppingPage *page)
 }
 
 static void
-serves_changed (GObject *object, GParamSpec *pspec, GrShoppingPage *page)
+yield_changed (GObject *object, GParamSpec *pspec, GrShoppingPage *page)
 {
-        GrRecipeSmallTile *tile = GR_RECIPE_SMALL_TILE (object);
+        GrShoppingTile *tile = GR_SHOPPING_TILE (object);
         GrRecipeStore *store;
         GrRecipe *recipe;
-        int serves;
+        double yield;
 
-        recipe = gr_recipe_small_tile_get_recipe (tile);
-        serves = gr_recipe_small_tile_get_serves (tile);
+        recipe = gr_shopping_tile_get_recipe (tile);
+        yield = gr_shopping_tile_get_yield (tile);
 
         store = gr_recipe_store_get ();
 
-        gr_recipe_store_add_to_shopping (store, recipe, serves);
+        gr_recipe_store_add_to_shopping (store, recipe, yield);
 }
 
 static void
 search_started (GrRecipeSearch *search,
-                GrShoppingPage     *page)
+                GrShoppingPage *page)
 {
         container_remove_all (GTK_CONTAINER (page->recipe_list));
         page->recipe_count = 0;
@@ -557,11 +570,11 @@ search_hits_added (GrRecipeSearch *search,
         for (l = hits; l; l = l->next) {
                 GrRecipe *recipe = l->data;
                 GtkWidget *tile;
-                int serves;
+                double yield;
 
-                serves = gr_recipe_store_get_shopping_serves (store, recipe);
-                tile = gr_recipe_small_tile_new (recipe, serves);
-                g_signal_connect (tile, "notify::serves", G_CALLBACK (serves_changed), page);
+                yield = gr_recipe_store_get_shopping_yield (store, recipe);
+                tile = gr_shopping_tile_new (recipe, yield);
+                g_signal_connect (tile, "notify::yield", G_CALLBACK (yield_changed), page);
                 gtk_container_add (GTK_CONTAINER (page->recipe_list), tile);
                 page->recipe_count++;
         }
@@ -581,7 +594,7 @@ search_hits_removed (GrRecipeSearch *search,
                 GrRecipe *recipe;
 
                 tile = gtk_bin_get_child (GTK_BIN (item));
-                recipe = gr_recipe_small_tile_get_recipe (GR_RECIPE_SMALL_TILE (tile));
+                recipe = gr_shopping_tile_get_recipe (GR_SHOPPING_TILE (tile));
                 if (g_list_find (hits, recipe)) {
                         gtk_container_remove (GTK_CONTAINER (page->recipe_list), item);
                         page->recipe_count--;
@@ -619,7 +632,7 @@ clear_list (GrShoppingPage *page)
         gr_window_go_back (GR_WINDOW (window));
 }
 
-static GList *
+GList *
 get_recipes (GrShoppingPage *page)
 {
         GList *children, *l, *recipes;
@@ -630,7 +643,7 @@ get_recipes (GrShoppingPage *page)
                 GtkWidget *tile = gtk_bin_get_child (GTK_BIN (l->data));
                 GrRecipe *recipe;
 
-                recipe = gr_recipe_small_tile_get_recipe (GR_RECIPE_SMALL_TILE (tile));
+                recipe = gr_shopping_tile_get_recipe (GR_SHOPPING_TILE (tile));
                 recipes = g_list_append (recipes, g_object_ref (recipe));
         }
         g_list_free (children);
@@ -638,7 +651,7 @@ get_recipes (GrShoppingPage *page)
         return recipes;
 }
 
-static GList *
+GList *
 get_ingredients (GrShoppingPage *page)
 {
         GList *children, *l, *ingredients;
@@ -661,7 +674,7 @@ get_ingredients (GrShoppingPage *page)
         return ingredients;
 }
 
-static void
+void
 item_free (gpointer data)
 {
         ShoppingListItem *item = data;
@@ -692,81 +705,18 @@ print_list (GrShoppingPage *page)
         g_list_free_full (items, item_free);
 }
 
-
 static void
-file_chooser_response (GtkNativeDialog  *self,
-                       int               response_id,
-                       GrShoppingPage   *page)
+open_export_shopping_list_dialog (GrShoppingPage *page)
 {
-        if (response_id == GTK_RESPONSE_ACCEPT) {
-                GList *recipes, *items;
-                g_autoptr(GFile) file = NULL;
-                g_autofree char *text = NULL;
-
-                recipes = get_recipes (page);
-                items = get_ingredients (page);
-
-                text = gr_shopping_list_format (recipes, items);
-
-                g_list_free_full (recipes, g_object_unref);
-                g_list_free_full (items, item_free);
-
-                file = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (self));
-                g_file_replace_contents (file, text, -1, NULL, FALSE, 0, NULL, NULL, NULL);
-        }
-
-        gtk_native_dialog_destroy (self);
-}
-
-static void
-mail_done (GObject      *source,
-           GAsyncResult *result,
-           gpointer      data)
-{
-        GrShoppingPage *page = data;
-        g_autoptr(GError) error = NULL;
-
-        if (!gr_send_mail_finish (result, &error)) {
-                GObject *file_chooser;
+        GList *items;
+        items = get_ingredients (page);
+        if (!page->exporter) {
                 GtkWidget *window;
 
-                g_info ("Sending mail failed: %s", error->message);
-
                 window = gtk_widget_get_ancestor (GTK_WIDGET (page), GTK_TYPE_APPLICATION_WINDOW);
-                file_chooser = (GObject *)gtk_file_chooser_native_new (_("Save the shopping list"),
-                                                                       GTK_WINDOW (window),
-                                                                       GTK_FILE_CHOOSER_ACTION_SAVE,
-                                                                       _("Save"),
-                                                                       _("Cancel"));
-                gtk_native_dialog_set_modal (GTK_NATIVE_DIALOG (file_chooser), TRUE);
-
-                g_signal_connect (file_chooser, "response", G_CALLBACK (file_chooser_response), page);
-
-                gtk_native_dialog_show (GTK_NATIVE_DIALOG (file_chooser));
-                return;
+                page->exporter = gr_shopping_list_exporter_new (GTK_WINDOW (window));
         }
-}
-
-static void
-share_list (GrShoppingPage *page)
-{
-        GList *recipes, *items;
-        g_autofree char *text = NULL;
-        GtkWidget *window;
-
-        recipes = get_recipes (page);
-        items = get_ingredients (page);
-
-        text = gr_shopping_list_format (recipes, items);
-
-        window = gtk_widget_get_ancestor (GTK_WIDGET (page), GTK_TYPE_APPLICATION_WINDOW);
-
-        gr_send_mail (GTK_WINDOW (window),
-                      NULL, _("Shopping List"), text, NULL,
-                      mail_done, page);
-
-        g_list_free_full (recipes, g_object_unref);
-        g_list_free_full (items, item_free);
+        gr_shopping_list_exporter_export (page->exporter, items);
 }
 
 static void
@@ -850,7 +800,7 @@ gr_shopping_page_class_init (GrShoppingPageClass *klass)
 
         gtk_widget_class_bind_template_callback (widget_class, clear_list);
         gtk_widget_class_bind_template_callback (widget_class, print_list);
-        gtk_widget_class_bind_template_callback (widget_class, share_list);
+        gtk_widget_class_bind_template_callback (widget_class, open_export_shopping_list_dialog);
         gtk_widget_class_bind_template_callback (widget_class, row_activated);
         gtk_widget_class_bind_template_callback (widget_class, removed_row_activated);
 }
@@ -885,7 +835,7 @@ recipe_removed (GrShoppingPage *page,
         for (l = children; l; l = l->next) {
                 GtkWidget *tile = gtk_bin_get_child (GTK_BIN (l->data));
 
-                if (recipe == gr_recipe_small_tile_get_recipe (GR_RECIPE_SMALL_TILE (tile))) {
+                if (recipe == gr_shopping_tile_get_recipe (GR_SHOPPING_TILE (tile))) {
                         gtk_widget_destroy (GTK_WIDGET (l->data));
 
                         collect_ingredients (page);
@@ -904,21 +854,21 @@ recipe_added (GrShoppingPage *page,
 {
         GrRecipeStore *store;
         GList *children, *l;
-        int serves;
+        double yield;
 
         if (!gtk_widget_is_drawable (GTK_WIDGET (page)))
                 return;
 
         store = gr_recipe_store_get ();
 
-        serves = gr_recipe_store_get_shopping_serves (store, recipe);
+        yield = gr_recipe_store_get_shopping_yield (store, recipe);
 
         children = gtk_container_get_children (GTK_CONTAINER (page->recipe_list));
         for (l = children; l; l = l->next) {
                 GtkWidget *tile = gtk_bin_get_child (GTK_BIN (l->data));
 
-                if (recipe == gr_recipe_small_tile_get_recipe (GR_RECIPE_SMALL_TILE (tile))) {
-                        gr_recipe_small_tile_set_serves (GR_RECIPE_SMALL_TILE (tile), serves);
+                if (recipe == gr_shopping_tile_get_recipe (GR_SHOPPING_TILE (tile))) {
+                        gr_shopping_tile_set_yield (GR_SHOPPING_TILE (tile), yield);
                         break;
                 }
         }
@@ -927,8 +877,8 @@ recipe_added (GrShoppingPage *page,
         if (l == NULL) {
                 GtkWidget *tile;
 
-                tile = gr_recipe_small_tile_new (recipe, serves);
-                g_signal_connect (tile, "notify::serves", G_CALLBACK (serves_changed), page);
+                tile = gr_shopping_tile_new (recipe, yield);
+                g_signal_connect (tile, "notify::yield", G_CALLBACK (yield_changed), page);
                 gtk_container_add (GTK_CONTAINER (page->recipe_list), tile);
         }
 
