@@ -425,21 +425,6 @@ dialog:
         return FALSE;
 }
 
-void
-all_headers (GtkListBoxRow *row,
-             GtkListBoxRow *before,
-             gpointer       user_data)
-{
-        GtkWidget *header;
-
-        header = gtk_list_box_row_get_header (row);
-        if (header)
-                return;
-
-        header = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-        gtk_list_box_row_set_header (row, header);
-}
-
 #ifdef GDK_WINDOWING_WAYLAND
 typedef struct {
         GtkWindow *window;
@@ -646,7 +631,7 @@ remove_image (const char *path)
                 g_remove (path);
         }
         else {
-                g_print ("Not removing image %s", path);
+                g_debug ("Not removing image %s", path);
         }
 }
 
@@ -824,3 +809,14 @@ strv_remove (char       ***strv_in,
         g_free (*strv_in);
         *strv_in = strv;
 }
+
+const char *
+get_version (void)
+{
+        const char *p = strrchr (PACKAGE_VERSION, '.');
+
+        if (p && (atoi (p + 1) % 2 == 1))
+                return COMMIT_ID;
+        else
+                return PACKAGE_VERSION;
+}
