@@ -61,10 +61,6 @@ gboolean in_flatpak_sandbox (void);
 gboolean portal_available (GtkWindow  *window,
                            const char *portal);
 
-void all_headers (GtkListBoxRow *row,
-                  GtkListBoxRow *before,
-                  gpointer       user_data);
-
 typedef void (*WindowHandleExported) (GtkWindow  *window,
                                       const char *handle,
                                       gpointer    user_data);
@@ -87,4 +83,4 @@ void strv_remove (char       ***strv_in,
 void strv_prepend (char       ***strv_in,
                    const char   *s);
 
-
+const char *get_version (void);
