@@ -47,6 +47,7 @@
 #include "gr-utils.h"
 #include "gr-appdata.h"
 #include "gr-settings.h"
+#include "gr-shopping-list-exporter.h"
 
 
 struct _GrWindow
@@ -99,6 +100,9 @@ struct _GrWindow
         GList *shopping_done_list;
         char **removed_ingredients;
 
+        GtkWidget *export_done_revealer;
+        guint export_done_timeout_id;
+
         GObject *file_chooser;
         GrRecipeImporter *importer;
         GrRecipeExporter *exporter;
@@ -601,6 +605,10 @@ gr_window_finalize (GObject *object)
                 g_source_remove (self->shopping_done_timeout_id);
                 self->shopping_done_timeout_id = 0;
         }
+        if (self->export_done_timeout_id) {
+                g_source_remove (self->export_done_timeout_id);
+                self->export_done_timeout_id = 0;
+        }
 
         g_list_free (self->dialogs);
 
@@ -791,7 +799,9 @@ back_to_shopping (GrWindow *window)
 {
         GrRecipeStore *store;
         GList *l;
+        GList *items;
         int i;
+        GrShoppingListExporter *exporter = NULL;
 
         store = gr_recipe_store_get ();
 
@@ -799,6 +809,14 @@ back_to_shopping (GrWindow *window)
                 ShoppingListEntry *entry = l->data;
                 gr_recipe_store_add_to_shopping (store, entry->recipe, entry->yield);
         }
+        items = get_ingredients (GR_SHOPPING_PAGE (window->shopping_page));
+        if (!exporter) {
+                GtkWidget *shopping_window;
+
+                shopping_window = gtk_widget_get_ancestor (GTK_WIDGET (window->shopping_page), GTK_TYPE_APPLICATION_WINDOW);
+                exporter = gr_shopping_list_exporter_new (GTK_WINDOW (shopping_window));
+                do_undo_in_todoist (exporter, items);
+        }
         for (i = 0; window->removed_ingredients && window->removed_ingredients[i]; i++) {
                 gr_recipe_store_remove_shopping_ingredient (store, window->removed_ingredients[i]);
         }
@@ -824,6 +842,7 @@ done_shopping (GrWindow *window)
         GList *recipes, *l;
         GrRecipeStore *store;
         const char **removed_ingredients;
+        GrShoppingListExporter *exporter = NULL;
 
         store = gr_recipe_store_get ();
 
@@ -855,6 +874,42 @@ done_shopping (GrWindow *window)
         window->removed_ingredients = g_strdupv ((char **)removed_ingredients);
 
         gr_recipe_store_clear_shopping_list (store);
+        if (!exporter) {
+                GtkWidget *shopping_window;
+
+                shopping_window = gtk_widget_get_ancestor (GTK_WIDGET (window->shopping_page), GTK_TYPE_APPLICATION_WINDOW);
+                exporter = gr_shopping_list_exporter_new (GTK_WINDOW (shopping_window));
+                done_shopping_in_todoist (exporter);
+        }
+}
+
+
+static void
+close_export_done (GrWindow *window)
+{
+    if (window->export_done_timeout_id) {
+            g_source_remove (window->export_done_timeout_id);
+            window->export_done_timeout_id = 0;
+    }
+
+    gtk_revealer_set_reveal_child (GTK_REVEALER (window->export_done_revealer), FALSE);
+}
+
+static gboolean
+export_done_timeout (gpointer data)
+{
+        GrWindow *window = data;
+
+        close_export_done (window);
+
+        return G_SOURCE_REMOVE;
+}
+
+void
+gr_window_confirm_shopping_exported (GrWindow *window)
+{
+    gtk_revealer_set_reveal_child (GTK_REVEALER (window->export_done_revealer), TRUE);
+    window->export_done_timeout_id = g_timeout_add_seconds (10, export_done_timeout, window);
 }
 
 static void
@@ -941,6 +996,7 @@ gr_window_class_init (GrWindowClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrWindow, remind_label);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, shopping_added_revealer);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, shopping_done_revealer);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, export_done_revealer);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, sort_by_label);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, sort_by_name_button);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, sort_by_recency_button);
@@ -971,6 +1027,7 @@ gr_window_class_init (GrWindowClass *klass)
         gtk_widget_class_bind_template_callback (widget_class, back_to_shopping);
         gtk_widget_class_bind_template_callback (widget_class, make_save_sensitive);
         gtk_widget_class_bind_template_callback (widget_class, sort_clicked);
+        gtk_widget_class_bind_template_callback (widget_class, close_export_done);
 }
 
 static GtkClipboard *
